from odoo.tests import tagged
from odoo.tests.common import TransactionCase


@tagged('post_install', '-at_install')
class TestAccessControl(TransactionCase):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        Group = cls.env['res.groups']
        cls.group1 = Group.create({'name': 'group1'})
        cls.group2 = Group.create({'name': 'group2'})
        cls.group3 = Group.create({'name': 'group3'})
        cls.group4 = Group.create({'name': 'group4'})
        cls.env['ir.model.data'].create({
            'module': 'test',
            'name': 'xmlid_for_group4',
            'model': Group._name,
            'res_id': cls.group4.id,
        })

        cls.tmpl234 = cls.env['access.control.template'].create({
            'code': 'TMPL234',
            'name': 'TMPL TEST 234',
            'group_ids': [(6, 0, (cls.group2 + cls.group3 + cls.group4).ids)],
        })

        cls.tmpl34 = cls.env['access.control.template'].create({
            'code': 'TMPL34',
            'name': 'TMPL TEST 34',
            'group_ids': [(6, 0, (cls.group3 + cls.group4).ids)],
        })

        cls.user = cls.env['res.users'].create({
            'name': 'test-accesscontrol',
            'login': 'test-accesscontrol@odoo.com'
        })

    def resetWz(self):
        self.wzr = self.env['access.control.wizard'].create({
            'user_id': self.user.id,
            'reason': 'test-ACCESS-CONTROL',
        })
        self.wzr.next()

    def test_01_update_group(self):
        self.resetWz()
        # add group
        old = self.user.groups_id
        self.wzr.action_todo = 'group_add'
        self.wzr.group_id = self.group1
        self.wzr.simulate()
        self.wzr.apply_rights()
        new = self.user.groups_id
        self.assertEqual(set(old).symmetric_difference(set(new)), set(self.group1))

        self.resetWz()

        # remove added group
        old = self.user.groups_id
        self.wzr.action_todo = 'group_remove'
        self.wzr.group_id = self.group1
        self.wzr.simulate()
        self.wzr.apply_rights()
        new = self.user.groups_id
        self.assertEqual(set(old).symmetric_difference(set(new)), set(self.group1))

    def test_02_update_template(self):
        self.resetWz()
        # add tmpl
        old = self.user.groups_id
        self.wzr.action_todo = 'template_add'
        self.wzr.template_id = self.tmpl234
        self.wzr.simulate()
        self.wzr.apply_rights()
        new = self.user.groups_id
        self.assertEqual(set(old).symmetric_difference(set(new)), set(self.group2 + self.group3 + self.group4))

        self.resetWz()

        # remove template
        old = self.user.groups_id
        self.wzr.action_todo = 'template_remove'
        self.wzr.template_id = self.tmpl34
        self.wzr.simulate()
        self.wzr.apply_rights()
        new = self.user.groups_id
        self.assertEqual(set(old).symmetric_difference(set(new)), set(self.group3 + self.group4))

        self.resetWz()

        # reset tmpl
        old = self.user.groups_id
        self.wzr.action_todo = 'template_reset'
        self.wzr.template_id = self.tmpl234
        self.wzr.simulate()
        self.wzr.apply_rights()
        new = self.user.groups_id
        self.assertEqual(set(old).symmetric_difference(set(new)), set(self.group3 + self.group4))

    def test_03_multi_op(self):
        self.resetWz()
        old = self.user.groups_id

        self.wzr.action_todo = 'template_reset'
        self.wzr.template_id = self.tmpl234
        self.wzr.simulate()

        self.wzr.action_todo = 'template_remove'
        self.wzr.template_id = self.tmpl34
        self.wzr.simulate()

        self.wzr.action_todo = 'group_add'
        self.wzr.group_id = self.group3
        self.wzr.simulate()

        self.wzr.action_todo = 'template_add'
        self.wzr.template_id = self.tmpl234
        self.wzr.simulate()

        self.wzr.action_todo = 'group_remove'
        self.wzr.group_id = self.group2
        self.wzr.simulate()

        self.wzr.apply_rights()
        new = self.user.groups_id
        self.assertEqual(set(old).symmetric_difference(set(new)), set(self.group3 + self.group4))

    def test_04_apply_method(self):
        old = self.user.groups_id
        REASON = "That a test"
        self.env['access.control.template'].apply(
            self.user.login,
            [
                ('template_reset', self.tmpl234.code),
                ('template_remove', self.tmpl34.code),
                ('group_add', self.group3.id),
                ('template_add', self.tmpl234.code),
                ('group_remove', 'test.xmlid_for_group4'),
            ],
            REASON,
        )
        new = self.user.groups_id
        self.assertEqual(set(old).symmetric_difference(set(new)), set(self.group2 + self.group3))

        last_log = self.env['access.control.log'].search([], order='id desc', limit=1)
        self.assertEqual(last_log.reason, REASON)

    def test_05_multi_apply_method(self):
        REASON = "That a test on multi user"

        user1 = self.env['res.users'].create({
            'name': 'test-accesscontrol1',
            'login': 'test-accesscontrol1@odoo.com'
        })

        user2 = self.env['res.users'].create({
            'name': 'test-accesscontrol2',
            'login': 'test-accesscontrol2@odoo.com'
        })
        user2.action_create_employee()

        users = [self.user.login, user1.id, user2.id]
        res = self.env['access.control.template'].apply(
            users,
            ('group_add', 'test.xmlid_for_group4'),
            REASON,
        )
        self.assertEqual(len(res), len(users))

        last_logs = self.env['access.control.log'].search_count([('reason', '=', REASON)])
        self.assertEqual(last_logs, len(users))
